/*
 * rlm_item2username.c
 *		Translate any string attribute from the request packet
 *		to the User-Name attribute.
 *
 * Version:     $Id$
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Copyright 2000  The FreeRADIUS server project
 * Copyright 2000  Alan DeKok <aland@ox.org>
 */

static const char rcsid[] = "$Id$";

#include	"autoconf.h"
#include	"libradius.h"

#include	<sys/stat.h>

#include	<stdio.h>
#include	<stdlib.h>
#include	<string.h>
#include	<ctype.h>

#include	"radiusd.h"
#include	"modules.h"

typedef struct rlm_item2username_t {
        char    *item_name;     /* The attribute inside Access-Request
				   ie Calling-Station-Id */
        int     item_attr;	/* Attribute from the Dictionary */
	char	*default_value; /* Default value, if not found in request */
} rlm_item2username_t;

static const CONF_PARSER module_config[] = {

	{ "item-name",  PW_TYPE_STRING_PTR,
	  offsetof(rlm_item2username_t,item_name), NULL,  NULL},

	{ "default",  PW_TYPE_STRING_PTR,
	  offsetof(rlm_item2username_t,default_value), NULL,  "0000000"},

	{ NULL, -1, 0, NULL, NULL }
};


static void do_item2username(rlm_item2username_t *data, REQUEST *request)
{
        VALUE_PAIR      *namepair;
        VALUE_PAIR      *request_pairs;
        VALUE_PAIR      *item_vp;
	VALUE_PAIR	*new_username_vp;
	char	*new_item;

	request_pairs = request->packet->vps;

	/*
	 *  Find item Value or use default
	 */

	item_vp = pairfind(request_pairs, data->item_attr);
	if (item_vp != NULL ){
	    DEBUG2("  item2username: item-name %s = %s", data->item_name,item_vp->strvalue);
	    new_item=item_vp->strvalue;
	} else {
	    DEBUG2("  item2username: Could not find item named %s in request, using default = %s", data->item_name, data->default_value);
	    new_item = data->default_value;
	}

	/*
	 *  Replace User-Name valuepair, or create new
	 */
	
	namepair = pairfind(request_pairs, PW_USER_NAME);
	if (namepair != NULL ) {
	    DEBUG2("  item2username: Original VALUEPAIR User-Name = %s ", namepair->strvalue);
	    strNcpy(namepair->strvalue, new_item, sizeof(namepair->strvalue));
	    namepair->length = strlen(namepair->strvalue);
	} else {
            DEBUG2("  item2username: No User-Name in original VALUEPAIR, creating.");
	    new_username_vp = paircreate(PW_USER_NAME, PW_TYPE_STRING);
		if (!new_username_vp) {
		    radlog(L_ERR|L_CONS, "no memory");
		    exit(1);
		}
	    strNcpy(new_username_vp->strvalue, data->default_value, sizeof(new_username_vp->strvalue));
	    new_username_vp->length = strlen(new_username_vp->strvalue);
	    DEBUG2("  New VALUEPAIR created, adding");
	    pairadd(&(request_pairs), new_username_vp);
        }

	namepair = pairfind(request_pairs, PW_USER_NAME);
	if (namepair != NULL ) {
	    DEBUG2("  item2username: New VALUEPAIR User-Name = %s ", namepair->strvalue);
	} else {
	    radlog(L_ERR|L_CONS,"ERROR! No User-Name VALUEPAIR");
	    exit(1);
	}

        /*
         *      We should have a User-Name attribute now.
         */
        if (request->username == NULL) {
                request->username = pairfind(request_pairs, PW_USER_NAME);
        }

}

/*
 *	Initialize.
 */
static int item2username_instantiate(CONF_SECTION *conf, void **instance)
{
	int	rcode;
	rlm_item2username_t *data;
	DICT_ATTR	*dattr;

	/*
	 *	Allocate room to put the module's instantiation data.
	 */
	data = (rlm_item2username_t *) rad_malloc(sizeof(*data));
	memset(data, 0, sizeof(*data));

	/*
	 *	Read this modules configuration data.
	 */
        if (cf_section_parse(conf, data, module_config) < 0) {
		free(data);
                return -1;
        }

	/*
	 *	Item-Name must be defined
	 */

        if (!data->item_name || !strlen(data->item_name)){
                radlog(L_ERR, "rlm_item2username: Item name not defined");
               /* checkval_detach(data); */
                return -1;
        }

        /*
         *      Discover the attribute number of the item-name
         */
        dattr = dict_attrbyname(data->item_name);
        if (!dattr) {
                radlog(L_ERR, "rlm_iterm2username: No such attribute %s",
                       data->item_name);
                /* checkval_detach(data); */
                return -1;
        }

	if (dattr->type != PW_TYPE_STRING) {
		radlog(L_ERR, "rlm_iterm2username: item-name attribute must be type STRING");
		/* checkval_detach(data); */
		return -1;
	}

	data->item_attr = dattr->attr;

	/*
	 *	Save the instantiation data for later.
	 */
	*instance = data;

	return 0;
}

/*
 *	Preprocess a request.
 */
static int item2username_authorize(void *instance, REQUEST *request)
{
	rlm_item2username_t *data = (rlm_item2username_t *) instance;

	do_item2username(data, request);

	return RLM_MODULE_OK; /* Meaning: try next authorization module */
}

/*
 *	Preprocess a request before accounting
 */
static int item2username_preaccounting(void *instance, REQUEST *request)
{
	rlm_item2username_t *data = (rlm_item2username_t *) instance;

	/*
	 *  Ensure that we have the SAME user name for both
	 *  authentication && accounting.
	 */
	do_item2username(data, request);

	return RLM_MODULE_OK;
}

/*
 *      Clean up the module's instance.
 */
static int item2username_detach(void *instance)
{
	rlm_item2username_t *data = (rlm_item2username_t *) instance;

        if (data->item_name)
                free((char *)data->item_name);

	free(data);

	return 0;
}

/* globally exported name */
module_t rlm_item2username = {
	"item2username",
	0,			/* type: reserved */
	NULL,			/* initialization */
	item2username_instantiate,	/* instantiation */
	{
		NULL,			/* authentication */
		item2username_authorize,	/* authorization */
		item2username_preaccounting, /* pre-accounting */
		NULL,			/* accounting */
		NULL,			/* checksimul */
		NULL,			/* pre-proxy */
		NULL,			/* post-proxy */
		NULL			/* post-auth */
	},
	item2username_detach,	/* detach */
	NULL,			/* destroy */
};

